﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using Powell.Teams.PublicApi.WebClient.Models;

namespace Powell.Teams.PublicApi.WebClient.Controllers
{
    using System.Net.Http;
    using System.Text;

    using Microsoft.AspNetCore.Http;
    using Microsoft.Extensions.Configuration;

    using Newtonsoft.Json;
    using Newtonsoft.Json.Linq;

    using Powell.Teams.PublicApi.WebClient.Helpers;

    public class HomeController : Controller
    {
        private readonly ILogger<HomeController> _logger;

        public readonly IConfiguration Configuration;

        public HomeController(ILogger<HomeController> logger, IConfiguration configuration)
        {
            _logger = logger;
            Configuration = configuration;
        }
        public async Task<IActionResult> Index()
        {
            _logger.LogInformation("Home Index");

            string directoryID = Environment.GetEnvironmentVariable("TenantId");
            string clientID = Environment.GetEnvironmentVariable("ClientId");
            string authorizationUrl = Environment.GetEnvironmentVariable("AuthorizationUrl");

            AzureTokenModel azureToken = null;

            string sessionToken = HttpContext.Session.GetString(Constants.SessionKeys.AzureToken);
            if (!string.IsNullOrWhiteSpace(sessionToken))
            {
                azureToken = JsonConvert.DeserializeObject<AzureTokenModel>(sessionToken);
            }

            if (azureToken == null || azureToken.access_token == null)
            {
                string loginUrl = string.Format("https://login.microsoftonline.com/{0}/oauth2/authorize?client_id={1}&response_type=code&redirect_uri={2}&response_mode=query&scope={3}&state={4}",
                    directoryID,
                    clientID,
                    authorizationUrl,
                    Constants.TargetResource.OfficeGraph + Constants.GraphAPIAuthorizations.Default,
                    Guid.NewGuid().ToString());
                _logger.LogInformation("loginUrl : " + loginUrl);

                return Redirect(loginUrl);
            }
            else
            {
                _logger.LogInformation("token : " + azureToken.access_token);
            }

            return View(await GetSampleDataToCreateTeam());
        }



        public async Task<ApiTeamCreationModel> GetSampleDataToCreateTeam()
        {
            ApiTeamCreationModel teamCreationModel = new ApiTeamCreationModel();
            string sessionToken = HttpContext.Session.GetString(Constants.SessionKeys.AzureToken);
            if (!string.IsNullOrWhiteSpace(sessionToken))
            {
                AzureTokenModel azureToken = JsonConvert.DeserializeObject<AzureTokenModel>(sessionToken);

                var httpClient = new HttpClient();
                var apiCaller = new ApiCallHelper(httpClient);
                JObject templateJson = await apiCaller.WebApiGetCallAsync(
                              $"{Environment.GetEnvironmentVariable("ApiUrl")}/beta/templates?$select=Id",
                              azureToken.access_token);

                ApiTemplateModel templateModel = new ApiTemplateModel();

                if (templateJson != null)
                {
                    templateModel = JsonConvert
                        .DeserializeObject<OperationResult<List<ApiTemplateModel>>>(templateJson.ToString())
                        .Result
                        .FirstOrDefault();
                }

                teamCreationModel = new ApiTeamCreationModel()
                {
                    TeamTemplateId = templateModel.Id,
                    Name = $"Team created from web client - {Guid.NewGuid()}",
                    Description = "Description",
                    Visibility = "Private",
                };
            }

            return teamCreationModel;
        }


        public async Task<IActionResult> GetTeams()
        {
            JObject jobject = null;
            string sessionToken = HttpContext.Session.GetString(Constants.SessionKeys.AzureToken);
            if (!string.IsNullOrWhiteSpace(sessionToken))
            {
                AzureTokenModel azureToken = JsonConvert.DeserializeObject<AzureTokenModel>(sessionToken);

                var httpClient = new HttpClient();
                var apiCaller = new ApiCallHelper(httpClient);
                jobject = await apiCaller.WebApiGetCallAsync(
                              $"{Environment.GetEnvironmentVariable("ApiUrl")}/beta/teams",
                              azureToken.access_token);
            }

            return View(jobject);
        }


        public async Task<IActionResult> GetNamingConventions()
        {
            JObject jobject = null;
            string sessionToken = HttpContext.Session.GetString(Constants.SessionKeys.AzureToken);
            if (!string.IsNullOrWhiteSpace(sessionToken))
            {
                AzureTokenModel azureToken = JsonConvert.DeserializeObject<AzureTokenModel>(sessionToken);

                var httpClient = new HttpClient();
                var apiCaller = new ApiCallHelper(httpClient);
                jobject = await apiCaller.WebApiGetCallAsync(
                              $"{Environment.GetEnvironmentVariable("ApiUrl")}/beta/namingconventions",
                              azureToken.access_token);
            }

            return View(jobject);
        }


        public async Task<IActionResult> GetTags()
        {
            JObject jobject = null;
            string sessionToken = HttpContext.Session.GetString(Constants.SessionKeys.AzureToken);
            if (!string.IsNullOrWhiteSpace(sessionToken))
            {
                AzureTokenModel azureToken = JsonConvert.DeserializeObject<AzureTokenModel>(sessionToken);

                var httpClient = new HttpClient();
                var apiCaller = new ApiCallHelper(httpClient);
                jobject = await apiCaller.WebApiGetCallAsync(
                              $"{Environment.GetEnvironmentVariable("ApiUrl")}/beta/tags",
                              azureToken.access_token);
            }

            return View(jobject);
        }


        public async Task<IActionResult> GetTemplates()
        {
            JObject jobject = null;
            string sessionToken = HttpContext.Session.GetString(Constants.SessionKeys.AzureToken);
            if (!string.IsNullOrWhiteSpace(sessionToken))
            {
                AzureTokenModel azureToken = JsonConvert.DeserializeObject<AzureTokenModel>(sessionToken);

                var httpClient = new HttpClient();
                var apiCaller = new ApiCallHelper(httpClient);
                jobject = await apiCaller.WebApiGetCallAsync(
                              $"{Environment.GetEnvironmentVariable("ApiUrl")}/beta/templates",
                              azureToken.access_token);
            }

            return View(jobject);
        }

        [HttpPost]
        public async Task<IActionResult> CreateTeam(ApiTeamCreationModel teamCreationModel)
        {
            JObject jobject = null;
            string sessionToken = HttpContext.Session.GetString(Constants.SessionKeys.AzureToken);
            if (!string.IsNullOrWhiteSpace(sessionToken))
            {
                AzureTokenModel azureToken = JsonConvert.DeserializeObject<AzureTokenModel>(sessionToken);

                ApiTeamCreationModel creationModelFull = new ApiTeamCreationModel();
                creationModelFull.Name = teamCreationModel.Name;
                creationModelFull.Description = teamCreationModel.Description;
                creationModelFull.TeamTemplateId = teamCreationModel.TeamTemplateId;
                creationModelFull.Visibility = teamCreationModel.Visibility;

                var httpClient = new HttpClient();
                var apiCaller = new ApiCallHelper(httpClient);
                jobject = await apiCaller.WebApiPostCallAsync(
                              $"{Environment.GetEnvironmentVariable("ApiUrl")}/beta/teams",
                              azureToken.access_token,
                              creationModelFull);
            }

            return View(jobject);
        }

        public IActionResult Privacy()
        {
            return View();
        }

        [ResponseCache(Duration = 0, Location = ResponseCacheLocation.None, NoStore = true)]
        public IActionResult Error()
        {
            return View(new ErrorViewModel { RequestId = Activity.Current?.Id ?? HttpContext.TraceIdentifier });
        }
    }
}
