﻿using System;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace Powell.Teams.PublicApi.WebClient.Helpers
{
    using System.IO;
    using System.Net.Http;
    using System.Net.Http.Headers;
    using System.Text;

    using Microsoft.AspNetCore.Http;
    using Microsoft.Extensions.Logging;

    public class GraphAPIHelper
    {
        public static async Task<string> GetAzureToken(
            string code,
            ISession session,
            ILogger logger,
            string resource = Constants.TargetResource.OfficeGraph)
        {
            string token = string.Empty;
            try
            {
                logger.LogInformation("GetAzureToken start");

                string directoryID = Environment.GetEnvironmentVariable("TenantId");
                string clientID = Environment.GetEnvironmentVariable("ClientId");
                string clientSecret = Environment.GetEnvironmentVariable("ClientSecret");
                string authorizationUrl = Environment.GetEnvironmentVariable("AuthorizationUrl");

                HttpClient client = new HttpClient();
                var content = new FormUrlEncodedContent(
                    new[]
                    {
                        new KeyValuePair<string, string>(
                            "grant_type",
                            "authorization_code"),
                        new KeyValuePair<string, string>(
                            "client_id",
                            clientID),
                        new KeyValuePair<string, string>(
                            "code",
                            code),
                        new KeyValuePair<string, string>(
                            "scope",
                            resource + Constants.GraphAPIAuthorizations.Default),
                        new KeyValuePair<string, string>(
                            "client_secret",
                            clientSecret),
                        new KeyValuePair<string, string>(
                            "redirect_uri",
                            authorizationUrl),
                    });

                content.Headers.ContentType = new MediaTypeHeaderValue("application/x-www-form-urlencoded");

                HttpResponseMessage response = client.PostAsync(
                        string.Format(
                            "https://login.microsoftonline.com/{0}/oauth2/v2.0/token",
                            directoryID),
                        content)
                    .Result;

                Stream receiveStream = response.Content.ReadAsStreamAsync()
                    .Result;
                StreamReader readStream = new StreamReader(
                    receiveStream,
                    Encoding.UTF8);
                token = readStream.ReadToEnd();

                logger.LogInformation("GetAzureToken response stream : " + token);
                byte[] bytes = Encoding.ASCII.GetBytes(token);
                session.Set(Constants.SessionKeys.AzureToken, bytes);
            }
            catch (Exception ex)
            {
                logger.LogInformation(
                    ex,
                    "GraphAPIHelper.GetAzureToken");
            }

            return token;
        }
    }
}
