﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace Powell.Teams.PublicApi.WebClient.Models
{
    /// <summary>
    ///     Describes the result of a server-side operation to the client-side code.
    /// </summary>
    /// <typeparam name="T">The type of the data returned by the operation.</typeparam>
    public class OperationResult<T>
    {
        /// <summary>
        ///     Initializes a new instance of the <see cref="OperationResult{T}" /> class.
        /// </summary>
        public OperationResult()
        {
        }

        /// <summary>
        ///     Initializes a new instance of the <see cref="OperationResult{T}" /> class.
        /// </summary>
        /// <param name="result">The result.</param>
        /// <param name="errorMessage">The error message.</param>
        /// <param name="isSucceed">If set to <c>true</c> [is succeed].</param>
        /// <param name="errorCode">The error code.</param>
        public OperationResult(T result, string errorMessage = null, bool isSucceed = true, int errorCode = 0)
        {
            this.Result = result;
            this.ErrorMessage = errorMessage;
            this.IsSucceed = isSucceed;
            this.ErrorCode = errorCode;
        }

        /// <summary>
        ///     Gets or sets the CorrelationId.
        /// </summary>
        /// <value>
        ///     The error message.
        /// </value>
        public Guid? CorrelationId { get; set; }

        /// <summary>
        ///     Gets or sets the internal error code.
        /// </summary>
        /// <value>
        ///     The error message.
        /// </value>
        public int ErrorCode { get; set; }

        /// <summary>
        ///     Gets or sets the error message.
        /// </summary>
        /// <value>
        ///     The error message.
        /// </value>
        public string ErrorMessage { get; set; }

        /// <summary>
        ///     Gets or sets a value indicating whether this instance is succeed.
        /// </summary>
        /// <value>
        ///     <c>true</c> if this instance is succeed; otherwise, <c>false</c>.
        /// </value>
        public bool IsSucceed { get; set; }

        /// <summary>
        ///     Gets or sets the result.
        /// </summary>
        /// <value>
        ///     The result.
        /// </value>
        public T Result { get; set; }
    }
}